cbuffer LevelParameters
{
  float4x4 projectionMatrix;
  float2 oneOverTextureSize;
  float2 coarseRatio;
};

cbuffer UpsampleParameters
{
  float2 sourceOrigin;
  float2 updateSize;
  float2 destinationOffset;
  float noData;
  bool enableNoData;
};

struct VertexOut
{
  float4 Position : SV_Position;
  float2 TextureCoordinates : TEXCOORD0;
};

float2 convertTexCoords(float2 texCoords)
{
#ifdef COGS_SHADER_GL
  return texCoords;
#else
  return float2(0, 1) + float2(1, -1) * texCoords;
#endif
}

VertexOut main(float2 Position : POSITION0)
{
  VertexOut Out = (VertexOut)0;

  float2 scaledPosition = Position * updateSize;

  Out.Position = mul(float4(scaledPosition + destinationOffset, 0.0, 1.0), projectionMatrix);
  Out.TextureCoordinates = convertTexCoords((scaledPosition * coarseRatio + sourceOrigin) * oneOverTextureSize);

  return Out;
}
