Texture2D bakedTable;
Texture2D source;

int fourierPass;

#ifdef DOUBLE
#define Value float4
#define Swizzle
#else
#define Value float2
#define Swizzle .xy
#endif

float2 complexMul(float2 a, float2 b)
{
  return float2(a.x * b.x - a.y * b.y,
                a.x * b.y + a.y * b.x);
}

float4 complexMul(float2 a, float4 b)
{
  return float4(a.x * b.x - a.y * b.y,
                a.x * b.y + a.y * b.x,
                a.x * b.z - a.y * b.w,
                a.x * b.w + a.y * b.z);
}

Value main(float4 ScreenPos : SV_Position) : SV_Target
{
  int2 ij = int2(ScreenPos.xy);

  float4 ti = bakedTable.Load(int3(ij.x, fourierPass, 0));
  float4 tj = bakedTable.Load(int3(ij.y, fourierPass, 0));

  Value src00 = source.Load(int3(ti.x, tj.x, 0)) Swizzle;
  Value src01 = source.Load(int3(ti.y, tj.x, 0)) Swizzle;
  Value src10 = source.Load(int3(ti.x, tj.y, 0)) Swizzle;
  Value src11 = source.Load(int3(ti.y, tj.y, 0)) Swizzle;

  Value iA = src00 + complexMul(tj.zw, src10);
  Value iB = src01 + complexMul(tj.zw, src11);

  return iA + complexMul(ti.zw, iB);
}
